#!/usr/bin/env python
"""
check_osx_services.py (v. 1.1)

Copyright (c) 2008 Tim Wilson <wilson@visi.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

This script is typically run via NRPE and utilizes the 'serveradmin' command line 
tool that's included with OS X Server to check the status of various built-in 
services such as AFP, QuickTime Streaming Server, and DHCP. Since serveradmin 
requires root privileges, this script uses sudo to allow the nagios user to execute 
the script. See the sample nrpe.cfg file for instructions on adding the nagios
user to the sudoers file.

First add a check command to nrpe.cfg. For example:

command[check_afp]=sudo /opt/local/libexec/nagios/check_osx_services -s afp

The script will check against warning and critical thresholds for services where it
makes sense to do so. For OS 10.4 those services that accept -w and -c switches are:

afp, smb, qtss, ftp, and jabber -- number of current connections
dhcp -- number of current leases
print -- number of current print jobs
web -- total KBytes

The use of -w and -c switches for these services is optional. If they are not provided,
the check will simply return OK or CRITICAL based on whether the service is running.

Typical usage:

$ ./check_osx_services -s afp -w 10 -c 20
afp: Started - Uptime:  6 hrs 5.30 mins, 0 Connections

$ ./check_osx_service -s web
web: Started - 3519 Total requests, 15564 Total kB

$ ./check_osx_services -s swupdate
swupdate: Started - Uptime:  6 hrs 4.98 mins, Last check: Mon Jan 14 20:53:56 2008

To get a list of available services use the -l switch

$ ./check_osx_services -l
afp
appserver
dhcp
dirserv
... (and so on)
"""

from optparse import OptionParser
import sys
import os
import re
import time

timestamp = re.compile('(?P<time>^\d{4,4}-\d{2,2}-\d{2,2} \d{2,2}:\d{2,2}:\d{2,2}) -.*$')

def secs2days(delta):
    """
    Convert a number of seconds to a more friendly hours, mins, and secs formats.
    """
    dayString = ""
    days = delta/86400
    delta = delta - days*86400
    hours = delta/3600
    delta = delta - hours*3600
    mins = delta/60.0
    if days > 1:
        dayString += "%s days" % days
    elif days == 1:
        dayString += "%s day" % days
    if hours > 1:
        dayString += " %s hrs" % hours
    elif hours == 1:
        dayString += " %s hr" % hours
    if mins > 0:
        dayString += " %.2f mins" % mins
    return dayString
    
def returnCheck(info):
    """
    Take the dictionary containing the information from the serveradmin command
    and generate the proper check script output and exit code.
    """
    if info['serviceName'] in ['afp', 'smb', 'jabber', 'qtss']:
        status = "%s: Started - Uptime: %s, %s Connections" % \
            (info['serviceName'], info['uptime'], info['currentConnections'])
        if info.has_key('criticalValue'):
            if info['currentConnections'] >= info['criticalValue']:
                print status
                sys.exit(2)
            elif info['currentConnections'] >= info['warningValue']:
                print status
                sys.exit(1)
    elif info['serviceName'] == 'swupdate':
        status = "%s: Started - Uptime: %s, Last update: %s" % \
            (info['serviceName'], info['uptime'], time.asctime(info['lastChecktime']))
    elif info['serviceName'] == 'dhcp':
        status = "%s: Started - %s Leases" % (info['serviceName'], info['numDHCPLeases'])
        if info.has_key('criticalValue'):
            if info['numDHCPLeases'] >= info['criticalValue']:
                print status
                sys.exit(2)
            elif info['numDHCPLeases'] >= info['warningValue']:
                print status
                sys.exit(1)
    elif info['serviceName'] == 'ftp':
        status = "%s: Started - %s Connections, Anonymous access: %s" % \
            (info['serviceName'],
             info['realConnectionCount'] + info['anonymousConnectionCount'],
             info['anonymousAccessPermitted'])
        if info.has_key('criticalValue'):
            if info['realConnectionCount'] >= info['criticalValue']:
                print status
                sys.exit(2)
            elif info['realConnectionCount'] >= info['warningValue']:
                print status
                sys.exit(1)
    elif info['serviceName'] == 'print':
        status = "%s: Started - %s Current queues, %s Current jobs" % \
            (info['serviceName'], info['currentQueues'], info['currentJobs'])
        if info.has_key('criticalValue'):
            if info['currentJobs'] >= info['criticalValue']:
                print status
                sys.exit(2)
            elif info['currentJobs'] >= info['warningValue']:
                print status
                sys.exit(1)
    elif info['serviceName'] == 'web':
        status = "%s: Started - %s Total requests, %s Total kB" % \
            (info['serviceName'], info['totalRequests'], info['totalKBytes'])
        if info.has_key('criticalValue'):
            if info['totalKBytes'] >= info['criticalValue']:
                print status
                sys.exit(2)
            elif info['totalKBytes'] >= info['warningValue']:
                print status
                sys.exit(1)
    else:
        status = "%s: Started" % info['serviceName']
    print status
    sys.exit(0)

def main():
    usage = "%prog -s <service name> [-w <value> -c <value>] [-l]"
    parser = OptionParser(usage, version="%prog 1.0")
    parser.set_defaults(listServices=False)
    parser.add_option("-s", "--service", dest="serviceName",
                      help="Check the status of the given service.")
    parser.add_option("-w", "--warning",
                      dest="warningValue", type="int",
                      help="Set the warning threshold value.")
    parser.add_option("-c", "--critical",
                      dest="criticalValue", type="int",
                      help="Set the critical threshold value.")
    parser.add_option("-l", "--list", action="store_true", dest="listServices",
                      help="List all of the available services.")
    (options, args) = parser.parse_args()
    
    # If the -l option was specified, list the available services and exit.
    if options.listServices:
        print os.popen("/usr/sbin/serveradmin list").read()
        sys.exit(0)

    # Create a dictionary to hold the serveradmin output and seed it with some
    # of the check parameters.
    info = {'serviceName': options.serviceName}
    if options.warningValue:
        info['warningValue'] = options.warningValue
    if options.criticalValue:
        info['criticalValue'] = options.criticalValue
    if info.has_key('warningValue') and not info.has_key('criticalValue'):
        print "Error: Can't specify a warning threshold without a critical threshold."
        sys.exit(3)
    elif info.has_key('criticalValue') and not info.has_key('warningValue'):
        print "Error: Can't specify a critical threshold without a warning threshold."
        sys.exit(3)
    
    # Get the service info by executing the serveradmin command.
    cmd = "serveradmin fullstatus %s" % options.serviceName
    cmdoutput = [line.strip() for line in os.popen(cmd).readlines()]
    
    # Take the output from serveradmin and build a dictionary with the output
    for line in cmdoutput:
        # Go line by line and process the serveradmin output
        i, j = line.split('=')
        try:
            value = int(j)
        except ValueError:
            value = j.strip()[1:-1]
            if value == "NO":
                value = False
            elif value == "YES":
                value = True
            elif timestamp.match(value):
                m = timestamp.match(value)
                value = time.strptime(m.group('time'), "%Y-%m-%d %H:%M:%S")
        key = i.split(':')[-1].strip()
        info[key] = value
    info['checkTime'] = time.localtime()
    if info.has_key('error'):
        print "Invalid service name: %s" % options.serviceName
        sys.exit(2)
    if info['state'] == 'STOPPED' or info['state'] == 'UNKNOWN':
        print "%s: Stopped" % info['serviceName']
        sys.exit(2)
    elif info['state'] == 'RUNNING':
        if info.has_key('startedTime') and info['serviceName'] != 'ftp':
            # The check for ftp in the line above is because serveradmin doesn't show
            # a startedTime value even when the ftp server is started.
            uptime = time.mktime(info['checkTime']) - time.mktime(info['startedTime'])
            info['uptime'] = secs2days(int(uptime))
        if info.has_key('startTime'):
            uptime = time.mktime(info['checkTime']) - time.mktime(info['startTime'])
            info['uptime'] = secs2days(int(uptime))
        returnCheck(info)
    else:
        print "%s: Status Unknown" % info['serviceName']
        sys.exit(3)    

if __name__ == '__main__':
	main()